App.controller('Controller', function ($scope) {
    var _this = this;
    _this.CurrentVersion = APP_VERSION;

    _this.Auditoriums = [];
    for (var i = 0; i < 24; i++) {
        _this.Auditoriums.push({ AuditoriumNumber: i + 1 });
    }

    _this.Projectors = ['Barco DP4K-BLP', 'Barco Series 4', 'Christie Xenon', 'Christie RGB', 'NEC NP-NC204IL', 'NEC NP-02HD', 'SONY R320'];
    _this.MediaBlocks = ['Barco Alchemy', 'Dolby IMS 2000', 'Dolby IMS 3000'];
    _this.SoundProcessors = ['Barco APX AuroMax', 'Dolby CP650', 'Dolby CP650 Serial', 'Dolby CP750', 'Dolby CP850', 'Dolby CP950', 'QSC DCP 100', 'QSC DCP 300', 'QSC DPM 100', 'QSC DPM 300'];


    _this.hasChanged = function (config, savedConfig) {
        var changed = !angular.equals(config, savedConfig);
        return changed;
    };



    // onLoggedIn callback.  this is where we will read the config file
    jnrWebsocket.addOnLoggedInListener(function onLoggedIn() {
        console.log('onLoggedIn');

        // here we get the peers.  we will use the peers to connect to the other 
        // auditoriums.  it is likely that all or most of the jniors in a ibooth
        // complex are meant for this application.
        jnrWebsocket.getRegistryListing('peers', function (node, keys) {
            jnrWebsocket.readRegistryKeys(keys, function (keyName, value) {
                if (null != value) {
                    var ipAddress = value.split(' ')[0];
                    var series3 = -1 != keyName.indexOf('$1')
                        || -1 != keyName.indexOf('$2')
                        || -1 != keyName.indexOf('$3')
                        || -1 != keyName.indexOf('$4');
                    // ignore blank ip addresses or series 3 jniors
                    if ('0.0.0.0' != ipAddress && !series3) {
                        createPeerConnection(ipAddress);
                    }
                }
            });
        });

        jnrWebsocket.readRegistryKey('appdata/ibooth/tmsipaddress', function (keyName, value) {
            if (value && null != value) {
                _this.TmsIpAddress = value;
                $scope.$apply();
            }
        });

        jnrWebsocket.readRegistryKey('appdata/ibooth/tmsport', function (keyName, value) {
            if (value && null != value) {
                _this.TmsPort = value;
                $scope.$apply();
            }
        });


        peerConnections.push(new PeerConnection(jnrWebsocket, _this.Auditoriums));
    });



    _this.uploadBackground = function () {
        var $uploadFile = $('#background-file-upload');
        var fileCtrl = $uploadFile[0];
        fileCtrl.onchange = function (evt) {
            var file = evt.target.files[0];
            _this.uploadFile(file, "/flash/public/ibooth/background.png");
            $('#background-file-upload').val('');
        };
        fileCtrl.click();
    };



    _this.uploadLogo = function () {
        var $uploadFile = $('#logo-file-upload');
        var fileCtrl = $uploadFile[0];
        fileCtrl.onchange = function (evt) {
            var file = evt.target.files[0];
            _this.uploadFile(file, "/flash/public/ibooth/logo.png");
            $('#logo-file-upload').val('');
        };
        fileCtrl.click();
    };



    _this.uploadFile = function (localFile, remoteFile) {
        if (localFile) {
            // files shouldnt be above 100K
            if (localFile.size > 102400) {
                bootbox.alert({
                    className: 'bb-danger',
                    title: "Invalid File",
                    message: "Selected file is too large"
                });
                return;
            }

            bootbox.confirm({
                className: 'bb-primary',
                title: "Are you sure you want to load the selected file?",
                message: "Are you sure you want to upload '" + localFile.name + "'?",
                buttons: {
                    cancel: {
                        label: 'No',
                        className: 'btn-default'
                    },
                    confirm: {
                        label: 'Yes',
                        className: 'btn-primary'
                    }
                },
                callback: function (result) {
                    if (result) {
                        var r = new FileReader();
                        r.onload = function (e) {
                            var contents = e.target.result;

                            try {
                                // make sure the flash/public directory exists
                                jnrWebsocket.mkDir('flash/public/ibooth', function () {

                                    // make sure the flash/public/ibooth directory exists
                                    jnrWebsocket.mkDir('flash/public/ibooth', function () {

                                        jnrWebsocket.writeFile(remoteFile, contents, function () {
                                            bootbox.alert({
                                                className: 'bb-success',
                                                title: 'Upload Success!',
                                                message: '<b>' + localFile.name + "</b> uploaded as <b>" + remoteFile + '</b>'
                                            });
                                        });
                                    });
                                });

                            } catch (e) {
                                alert("Failed uploading the file" + e.toString());
                                return false;
                            }
                            return true;
                        };
                        r.readAsBinaryString(localFile);
                    }
                }
            });
        }
    };



    _this.saveTmsInfo = function () {
        jnrWebsocket.registryWrite('AppData/ibooth/TmsIpAddress', _this.TmsIpAddress);
        jnrWebsocket.registryWrite('AppData/ibooth/TmsPort', _this.TmsPort);
    };



    _this.fillProjectorIpAddresses = function () {
        var dialog = bootbox.dialog({
            className: 'bb-primary',
            title: 'Projector IP Auto Populate',
            message: '<form class="form-horizontal">\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Starting IP Address</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="text" id="starting-address" class="form-control" defaultValue="192.168.1.10" placeholder="192.168.1.10">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Ending IP Address</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="text" id="ending-address" class="form-control" placeholder="192.168.1.120">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Address Interval</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="number" id="address-interval" class="form-control" value="10">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '</form>',
            buttons: {
                confirm: {
                    label: 'OK',
                    className: 'btn-success',
                    callback: function () {
                        try {
                            var startingAddress = $("#starting-address").val();
                            if (!startingAddress) {
                                startingAddress = $("#starting-address").attr('placeholder');
                            }

                            var endingAddress = $("#ending-address").val();
                            if (!endingAddress) {
                                endingAddress = $("#ending-address").attr('placeholder');
                            }

                            var addressInterval = $("#address-interval").val();
                            addressInterval = parseInt(addressInterval);

                            //
                            // split the starting ip address into 4 octets.  make sure there are 
                            // 4 octets to validate the ip addresses.
                            var startingIp = startingAddress.split('.');
                            if (4 != startingIp.length) throw new Exception("Starting Address in incorect format");
                            var endingIp = endingAddress.split('.');
                            if (4 != endingIp.length) throw new Exception("Ending Address in incorect format");

                            // 
                            // create new addresses and assign them to peer connections
                            // peerConnections.forEach(function (peerConnection) {
                            for (var i = 0; i < peerConnections.length; i++) {
                                peerConnection = peerConnections[i];
                                if (peerConnection) {
                                    var auditoriumNumber = peerConnection.AuditoriumObject.AuditoriumNumber;
                                    if (auditoriumNumber) {
                                        var lastOctet = parseInt(startingIp[3]);
                                        lastOctet = (auditoriumNumber - 1) * addressInterval + lastOctet;
                                        if (lastOctet <= parseInt(endingIp[3])) {
                                            var ipAddress = [startingIp[0], startingIp[0], startingIp[0], lastOctet].join('.');
                                            peerConnection.AuditoriumObject.Config.ProjectorIpAddress = ipAddress;
                                        }
                                    }
                                }
                            }

                            $scope.$apply();
                        } catch (err) {
                            console.err(err);
                            alert(err);
                        }
                    }
                }
            }
        });
    };


    _this.fillMediaBlockIpAddresses = function () {
        var dialog = bootbox.dialog({
            className: 'bb-primary',
            title: 'Media Block IP Auto Populate',
            message: '<form class="form-horizontal">\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Starting IP Address</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="text" id="starting-address" class="form-control" defaultValue="192.168.1.10" placeholder="192.168.1.10">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Ending IP Address</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="text" id="ending-address" class="form-control" placeholder="192.168.1.120">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Address Interval</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="number" id="address-interval" class="form-control" value="10">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '</form>',
            buttons: {
                confirm: {
                    label: 'OK',
                    className: 'btn-success',
                    callback: function () {
                        try {
                            var startingAddress = $("#starting-address").val();
                            if (!startingAddress) {
                                startingAddress = $("#starting-address").attr('placeholder');
                            }

                            var endingAddress = $("#ending-address").val();
                            if (!endingAddress) {
                                endingAddress = $("#ending-address").attr('placeholder');
                            }

                            var addressInterval = $("#address-interval").val();
                            addressInterval = parseInt(addressInterval);

                            //
                            // split the starting ip address into 4 octets.  make sure there are 
                            // 4 octets to validate the ip addresses.
                            var startingIp = startingAddress.split('.');
                            if (4 != startingIp.length) throw new Exception("Starting Address in incorect format");
                            var endingIp = endingAddress.split('.');
                            if (4 != endingIp.length) throw new Exception("Ending Address in incorect format");

                            // 
                            // create new addresses and assign them to peer connections
                            // peerConnections.forEach(function (peerConnection) {
                            for (var i = 0; i < peerConnections.length; i++) {
                                peerConnection = peerConnections[i];
                                if (peerConnection) {
                                    var auditoriumNumber = peerConnection.AuditoriumObject.AuditoriumNumber;
                                    if (auditoriumNumber) {
                                        var lastOctet = parseInt(startingIp[3]);
                                        lastOctet = (auditoriumNumber - 1) * addressInterval + lastOctet;
                                        if (lastOctet <= parseInt(endingIp[3])) {
                                            var ipAddress = [startingIp[0], startingIp[0], startingIp[0], lastOctet].join('.');
                                            peerConnection.AuditoriumObject.Config.MediaBlockIpAddress = ipAddress;
                                        }
                                    }
                                }
                            }

                            $scope.$apply();
                        } catch (err) {
                            console.err(err);
                            alert(err);
                        }
                    }
                }
            }
        });
    };


    _this.fillSoundProcessorIpAddresses = function () {
        var dialog = bootbox.dialog({
            className: 'bb-primary',
            title: 'Sound Processor IP Auto Populate',
            message: '<form class="form-horizontal">\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Starting IP Address</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="text" id="starting-address" class="form-control" defaultValue="192.168.1.10" placeholder="192.168.1.10">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Ending IP Address</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="text" id="ending-address" class="form-control" placeholder="192.168.1.120">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '  <div class="form-group">\r\n' +
                '    <label class="col-sm-4 control-label">Address Interval</label>\r\n' +
                '    <div class="col-sm-4">\r\n' +
                '      <input type="number" id="address-interval" class="form-control" value="10">\r\n' +
                '    </div>\r\n' +
                '  </div>\r\n' +
                '</form>',
            buttons: {
                confirm: {
                    label: 'OK',
                    className: 'btn-success',
                    callback: function () {
                        try {
                            var startingAddress = $("#starting-address").val();
                            if (!startingAddress) {
                                startingAddress = $("#starting-address").attr('placeholder');
                            }

                            var endingAddress = $("#ending-address").val();
                            if (!endingAddress) {
                                endingAddress = $("#ending-address").attr('placeholder');
                            }

                            var addressInterval = $("#address-interval").val();
                            addressInterval = parseInt(addressInterval);

                            //
                            // split the starting ip address into 4 octets.  make sure there are 
                            // 4 octets to validate the ip addresses.
                            var startingIp = startingAddress.split('.');
                            if (4 != startingIp.length) throw new Exception("Starting Address in incorect format");
                            var endingIp = endingAddress.split('.');
                            if (4 != endingIp.length) throw new Exception("Ending Address in incorect format");

                            // 
                            // create new addresses and assign them to peer connections
                            // peerConnections.forEach(function (peerConnection) {
                            for (var i = 0; i < peerConnections.length; i++) {
                                peerConnection = peerConnections[i];
                                if (peerConnection) {
                                    var auditoriumNumber = peerConnection.AuditoriumObject.AuditoriumNumber;
                                    if (auditoriumNumber) {
                                        var lastOctet = parseInt(startingIp[3]);
                                        lastOctet = (auditoriumNumber - 1) * addressInterval + lastOctet;
                                        if (lastOctet <= parseInt(endingIp[3])) {
                                            var ipAddress = [startingIp[0], startingIp[0], startingIp[0], lastOctet].join('.');
                                            peerConnection.AuditoriumObject.Config.SoundProcessorIpAddress = ipAddress;
                                        }
                                    }
                                }
                            }

                            $scope.$apply();
                        } catch (err) {
                            console.err(err);
                            alert(err);
                        }
                    }
                }
            }
        });
    };


    var peerConnections = [];
    function createPeerConnection(ipAddress) {
        var newJnrWebsocket = new JnrWebsocket();
        var wsUri = 'ws://' + ipAddress;
        if (window.location.protocol == "https:")
            wsUri = 'wss://' + ipAddress;
        newJnrWebsocket.IpAddress = ipAddress;
        newJnrWebsocket.connect(wsUri);
        newJnrWebsocket.addOnLoggedInListener(function onLoggedIn() {
            console.log(newJnrWebsocket.IpAddress + ':onLoggedIn');

            peerConnections.push(new PeerConnection(newJnrWebsocket, _this.Auditoriums));
        });
    }



    function PeerConnection(jnrWebsocket, auditoriums) {
        var _this = this;
        _this.jnrWebsocket = jnrWebsocket;
        _this.Auditoriums = auditoriums;
        _this.AuditoriumObject = {
            PeerConnection: this, InUse: false, Config: null, SavedConfig: null, hasChanged: function () {
                return angular.eq
            }
        };


        _this.saveDialog = null;
        _this.save = function (config) {
            _this.saveDialog = bootbox.dialog({
                className: 'bb-primary',
                title: 'Save Changes',
                message: 'Are you sure you want to save your changes?',
                buttons: {
                    confirm: {
                        label: 'OK',
                        className: 'btn-success',
                        callback: function () {
                            _this.jnrWebsocket.postMessage(APP_ID, { Message: 'config.save', Config: config });
                        }
                    },
                    cancel: {
                        label: 'Cancel',
                        className: 'btn-default',
                        callback: function () {
                            _this.saveDialog.modal('hide');
                        }
                    }
                }
            });
        };


        _this.reboot = function (config) {
            _this.saveDialog = bootbox.dialog({
                className: 'bb-warning',
                title: 'Reboot',
                message: 'Are you sure you want to reboot the JNIOR in this auditorium?',
                buttons: {
                    confirm: {
                        label: 'OK',
                        className: 'btn-success',
                        callback: function () {
                            _this.jnrWebsocket.postMessage(APP_ID, { Message: 'reboot.execute', Config: config });
                        }
                    },
                    cancel: {
                        label: 'Cancel',
                        className: 'btn-default',
                        callback: function () {
                            _this.saveDialog.modal('hide');
                        }
                    }
                }
            });
        };


        _this.jnrWebsocket.addOnMessageListener(function (jsonReply) {
            var message = jsonReply.Message;

            console.log(_this.jnrWebsocket.IpAddress + ":" + message);
            if ("config.update" === message) {
                configResponseReceived = true;

                if (_this.saveDialog && null != _this.saveDialog) {
                    _this.saveDialog.modal('hide');
                }

                _this.AuditoriumObject.Config = jsonReply.Data;
                _this.AuditoriumObject.SavedConfig = angular.copy(_this.AuditoriumObject.Config);
                var auditoriumNumber = _this.AuditoriumObject.Config.AuditoriumNumber;
                if (-1 == auditoriumNumber) {

                    //
                    // get and the new auditorium number and post the change so that applciation can consume it
                    var newAuditoriumNumber = prompt("The " + _this.jnrWebsocket.IpAddress + " unit is running the iBooth application "
                        + "but has not been assigned an auditorium number.  Please assign one now.");
                    if (newAuditoriumNumber) {
                        _this.jnrWebsocket.postMessage(APP_ID, { Message: 'config.setauditoriumnumber', Number: newAuditoriumNumber });
                    }

                } else {
                    _this.AuditoriumObject.AuditoriumNumber = auditoriumNumber;
                    _this.Auditoriums[auditoriumNumber - 1] = _this.AuditoriumObject;
                    _this.AuditoriumObject.InUse = true;

                }

                // alert(JSON.stringify(_this.AuditoriumObject, null, 2));
                $scope.$apply();
            }
        });


        //
        // set a config response timeout timer.  if the config is not received then we close the connection.
        var configResponseReceived = false;
        window.setTimeout(function () {
            if (!configResponseReceived) {
                _this.jnrWebsocket.close();
            }
        }, 5000);
        _this.jnrWebsocket.postMessage(APP_ID, { Message: 'config.get' });


    };
});